
class PGame {
  
  PStick p1 ;
  PStick p2 ;
  PBall ball ;
  PScorer sc1 ;
  PScorer sc2 ;
  
  int gStatus ;
  int STOPPED = 0 ;
  int PLAYING = 1 ;
  int PAUSED = 2 ;
  int FINISHED = 3 ;
  
  int sWidth    = 800 ;
  int sHeight   = 600 ;
  int lWidth    = 20 ;
  int lW        = 15 ; 
  int backColor = 30 ;
  int lineColor = 255 ;
  int fRate     = 60 ;
  int incSpeed  = 2 ; 
  int maxSpeed  = 8 ;
  int maxBallSpeed = 10 ;
  int minSpeed  = 4 ;
  int incY      = 3 ;
  int maxScore  = 15 ;
  int minX      = - 2 * lWidth ;
  int maxX      = sWidth + ( 2 + lWidth ) ;
  PFont font ;
  int fontColor = 255 ;
  String fontName = "Arial" ;
  PSound sound ;
    
  PGame ( int _width, int _height ) {
    sWidth = _width ;
    sHeight = _height ;
    
    fill ( color ( 255, 20 )) ;
    rect ( 0, 0, _width, _height ) ;
    font = createFont ( fontName, 40, true ) ; 
    paintField () ;
    minX   = 0 ;
    maxX   = sWidth ;
    sound  = new PSound () ;
    p1     = new PStick ( lWidth, lWidth * 6, lWidth, _height / 2, incSpeed, maxSpeed ) ; 
    p2     = new PStick ( lWidth, lWidth * 6, _width - ( 2 * lWidth ), _height / 2, incSpeed, maxSpeed ) ;   
    ball   = new PBall ( lWidth, false, true, maxBallSpeed, minSpeed ) ;
    sc1    = new PScorer ((( _width / 2 ) - ( 6 * lWidth )), 6 * lWidth ) ;
    sc2    = new PScorer ((( _width / 2 ) + ( 3 * lWidth )), 6 * lWidth ) ;
    gStatus = STOPPED ;
    printText ( "Pulsa una tecla", 40, ( _width / 2 ) - 100, _height / 2, 3 ) ;
  }
  
  void prepareGame () {
    gStatus = STOPPED ;
    sc1.resetScore () ;
    sc2.resetScore () ;
    
    p1.setPosition ( new PVector ( lWidth, sHeight / 2 )) ;
    p1.setVelocity ( new PVector ( 0, 0 )) ;
    p2.setPosition ( new PVector ( sWidth - ( 2 * lWidth ), sHeight / 2 )) ;
    p2.setVelocity ( new PVector ( 0, 0 )) ;  
    ball.setPosition ( new PVector ( sWidth / 2, sHeight / 2 )) ;
    ball.setVelocity ( new PVector ( 0, 0 )) ;
    
    update () ;
    printText ( "Pulsa una tecla", 40, ( sWidth / 2 ) - 100, sHeight / 2, 3 ) ;
 }

 void startGame () {
   ball.kickOff () ;
   sound.playSound ( "pip.ogg" ) ; 
   gStatus = PLAYING ;
 }
 
 void printText ( String _myString, int _size, int _x, int _y, int _z ) {
    fill ( fontColor ) ;
    textFont ( font, _size ) ;
    text ( _myString, _x, _y, _z ) ;
 }
 
 void getMouse ()
 {
  if (( mouseY - pmouseY ) > incY ) {
   p1.goDown () ;
  }
  else {
    if (( pmouseY - mouseY ) > incY ) {
      p1.goUp () ;
    }
  } 
 }
 
 void getKeyboard () {
   if ( key == CODED ) {
    if ( keyCode == UP ) {
      p1.goUp () ;
     } else if ( keyCode == DOWN ) {
       p1.goDown () ;
     } 
   }
 }
 
 void paintField () {   
   background ( backColor ) ;
   // Linea discontinua
   float step = lWidth ;  
   for ( int i = 0; i <= step ; i++ ) {
     float x = lerp (( width - lW )/ 2, ( width - lW ) / 2, i/step ) + step ;
     float y = lerp ( lW, height, i/step ) ;
     rect ( x, y, lW, lW ) ;
   }
   rect ( 0, 0, width, lW ) ;
   rect ( 0, height - lW, width, lW ) ;
 }
   
 void endGame ( String _myString ) {
   gStatus = FINISHED ;      
   fill ( color ( 255, 20 )) ;
   rect ( 0, 0, sWidth, sHeight ) ;
   printText ( _myString, 60, ( sWidth / 2 ) - 140, sHeight / 2, 3 ) ;
 }
 
 int status () {
   return gStatus ;
 }
 
 void update () {   
   if ( gStatus == PLAYING ) {
     if ( ball.getX () < minX ) {
       ball.outOfBounds () ;
       sc2.incScore () ;
       if ( sc2.getScore () >= maxScore ) {
         endGame ( "Gano YO!!!" ) ;         
         return ;
       }
       ball.kickOff () ; 
     }
     else {
       if ( ball.getX () > maxX ) {
         ball.outOfBounds () ;
         sc1.incScore () ;
         if ( sc1.getScore () >= maxScore ) {
           endGame ( "Jugador 1 Gana" ) ;           
           return ;
         }
         ball.kickOff () ;
       }
     }
     
     if (( p1.hitObject ( ball ) > 0 ) && (( p1.getX () + ( p1.getWidth () / 2 )) < ball.getX ()) && ( ball.getVelocity().x < 0 )) {
       p1.hit ( ball ) ;
     }
     if (( p2.hitObject ( ball ) > 0 ) && (( p2.getX () - ( p2.getWidth () / 2 )) > ball.getX ()) && ( ball.getVelocity().x > 0 )) {
       p2.hit ( ball ) ;
     }
     
     p2.follow    ( ball ) ;     
   }
   paintField   () ;  
   p1.update    () ;
   p2.update    () ;
   ball.update  () ;
   p1.display   () ;
   p2.display   () ;
   ball.display () ;
   sc1.display  () ;
   sc2.display  () ;
 }
}
